/* (C) 2021 YiRing, Inc. */
package com.yiring.app.config;

import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import javax.validation.ConstraintViolationException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * 全局错误处理
 *
 * @author fangzhimin
 * 2017年11月30日 上午11:36:31
 */
@Slf4j
@ControllerAdvice
@ResponseBody
public class GlobalExceptionHandler {

    /**
     * 参数校验异常
     *
     * @param e 异常信息
     * @return 统一的校验失败信息 {@link Status#BAD_REQUEST
     */
    @ExceptionHandler(
        value = { BindException.class, MethodArgumentNotValidException.class, ConstraintViolationException.class }
    )
    public Result<String> bindErrorHandler(Exception e) {
        String error = "未知参数校验错误";

        if (e instanceof ConstraintViolationException) {
            ConstraintViolationException ex = (ConstraintViolationException) e;
            error = ex.getConstraintViolations().iterator().next().getMessage();
        } else {
            BindingResult result = null;
            if (e instanceof MethodArgumentNotValidException) {
                result = ((MethodArgumentNotValidException) e).getBindingResult();
            } else if (e instanceof BindException) {
                result = ((BindException) e).getBindingResult();
            }

            if (result != null) {
                error = result.getAllErrors().iterator().next().getDefaultMessage();
            }
        }

        return Result.no(Status.BAD_REQUEST, error);
    }

    /**
     * 不支持的HttpMethod异常
     *
     * @param e 异常信息
     * @return 异常信息反馈 {@link Status#METHOD_NOT_ALLOWED
     */
    @ExceptionHandler(value = HttpRequestMethodNotSupportedException.class)
    public Result<String> httpRequestMethodNotSupportedErrorHandler(Exception e) {
        return Result.no(Status.METHOD_NOT_ALLOWED, e.getMessage());
    }

    /**
     * 其他异常
     *
     * @param e 异常信息
     * @return 统一的500异常信息 {@link Status#INTERNAL_SERVER_ERROR
     */
    @ExceptionHandler(value = Exception.class)
    public Result<String> defaultErrorHandler(Exception e) {
        log.error(e.getMessage(), e);
        return Result.no(Status.INTERNAL_SERVER_ERROR, e);
    }
}
