/* (C) 2021 YiRing, Inc. */
package com.yiring.common.swagger;

import com.github.xiaoymin.knife4j.spring.extension.OpenApiExtensionResolver;
import com.yiring.common.core.Status;
import io.swagger.annotations.Api;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Profile;
import org.springframework.web.bind.annotation.RequestMethod;
import springfox.bean.validators.configuration.BeanValidatorPluginsConfiguration;
import springfox.documentation.builders.*;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.*;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2WebMvc;

/**
 * Swagger Config
 *
 * @author ifzm
 * @version 0.1
 * 2019/3/11 19:13
 */

@Slf4j
@Profile(value = { "default", "dev", "mock", "test", "preview" })
@EnableSwagger2WebMvc
@Configuration
@Import(BeanValidatorPluginsConfiguration.class)
public class SwaggerConfig {

    @Value("${spring.application.name}")
    String applicationName;

    @Value("${server.port}")
    String serverPort;

    @Resource
    OpenApiExtensionResolver openApiExtensionResolver;

    @Bean
    public Docket api() {
        return api(PathSelectors.any());
    }

    private Docket api(Predicate<String> paths) {
        log.info("API Doc: http://localhost:{}/doc.html", serverPort);

        String group = "default";
        return new Docket(DocumentationType.SWAGGER_2)
            .groupName(group)
            .apiInfo(apiInfo())
            .useDefaultResponseMessages(false)
            .globalOperationParameters(buildGlobalRequestParameters())
            .globalResponseMessage(RequestMethod.GET, buildGlobalResponseMessage())
            .globalResponseMessage(RequestMethod.POST, buildGlobalResponseMessage())
            .globalResponseMessage(RequestMethod.DELETE, buildGlobalResponseMessage())
            .globalResponseMessage(RequestMethod.PUT, buildGlobalResponseMessage())
            .select()
            .apis(RequestHandlerSelectors.withClassAnnotation(Api.class))
            .paths(paths)
            .build()
            .extensions(openApiExtensionResolver.buildExtensions(group));
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
            .title("Swagger 接口文档.")
            .description(applicationName)
            .version("1.0")
            .contact(new Contact("© YiRing", "https://yiring.com", "developer@yiring.com"))
            .build();
    }

    /**
     * 构建全局请求参数
     *
     * @return Token 等自定义全局参数/Header
     */
    private List<Parameter> buildGlobalRequestParameters() {
        List<Parameter> parameters = new ArrayList<>();
        parameters.add(
            new ParameterBuilder()
                .name("Authorization")
                .description("Auth Token")
                .required(true)
                .parameterType("header")
                .defaultValue("Basic Auth Token")
                .modelRef(new ModelRef("String"))
                .build()
        );
        return parameters;
    }

    /**
     * 构建全局响应消息
     *
     * @return 所有自定义状态码消息
     */
    private List<ResponseMessage> buildGlobalResponseMessage() {
        return Arrays
            .stream(Status.values())
            .map(status -> new ResponseMessageBuilder().code(status.value()).message(status.getReasonPhrase()).build())
            .collect(Collectors.toList());
    }
}
